from typing import Dict
from .Log import *
from .Job import Task, TaskUid


class JobsState:
    """
        represents a snapshot of the state of SGE jobs as seen by the SGE command "qstat -f -u \*"
    """
    tasks: Dict[TaskUid, Task]
    job_array_tasks: Dict[int, Dict[TaskUid, Task]]

    def __init__(self):
        self.tasks = {}  # list of jobs
        self.job_array_tasks = {}  # a dictionary of jobs for each job array, indexed by job array id
        self.m_queueMachines = {}    # list of queue machines such as allintel.q@simpatix10
        self.m_stateTime = None  # the time at which the state was snapshot

    def deleteAllJobs(self):
        self.tasks = {}
        self.job_array_tasks = {}

    def addTask(self, task: Task):
        task_uid = task.get_id()
        self.tasks[task_uid] = task
        if task_uid.is_job_array_element():
            tasks = self.job_array_tasks.get(task_uid.job_id)
            if tasks is None:
                tasks = {}
                self.job_array_tasks[task_uid.job_id] = tasks
            tasks[task_uid] = task

    def get_task(self, task_uid: TaskUid) -> Task:
        return self.tasks.get(task_uid)

    def get_job_array_tasks(self, job_array_id: int) -> Dict[TaskUid, Task]:
        return self.job_array_tasks.get(job_array_id)

    def setTime(self, stateTime):
        self.m_stateTime = stateTime

    def getTime(self):
        return self.m_stateTime

    def getJobsOnMachine(self, machineName):
        jobs_on_machine = {}
        for task_uid, task in self.tasks.items():
            for queueMachineName, numSlots in task.get_slots().items():
                jobMachineName = queueMachineName.split('@')[1]
                if jobMachineName == machineName:
                    jobs_on_machine[task_uid] = task
        return jobs_on_machine

    def getNumFreeSlotsOnQueueMachine(self, queueMachine):
        # logInfo('getNumFreeSlotsOnQueueMachine : looking for free slots on queuemachine %s' % queueMachine.getName())
        numUsedSlots = 0
        for job in self.tasks.values():
            numUsedSlotsByThisJob = job.get_slots().get(queueMachine.getName())
            if numUsedSlotsByThisJob is not None:
                # logInfo('getNumFreeSlotsOnQueueMachine : job %d uses %d slots' % (job.getId().asStr(), numUsedSlotsByThisJob))
                numUsedSlots += numUsedSlotsByThisJob
            else:
                None
                # logInfo('getNumFreeSlotsOnQueueMachine : job %d uses no slot' % job.getId().asStr())
        numFreeSlots = queueMachine.getNumSlots() - numUsedSlots
        assert numFreeSlots >= 0
        return numFreeSlots

    def addQueueMachine(self, queueMachine):
        self.m_queueMachines[queueMachine.getName()] = queueMachine

    def getQueueMachine(self, machineName):
        """
            finds the queue machine associated with a machine
        """
        queueMachine = None
        for qmName, qm in self.m_queueMachines.items():
            if qm.m_machineName == machineName:
                assert queueMachine is None  # to be sure that no more than one queue machine is on a given machine
                queueMachine = qm
        return queueMachine

    def getQueueMachines(self):
        return self.m_queueMachines

    def getPendingJobs(self):
        pendingJobs = {}
        for jobId, job in self.tasks.items():
            if job.is_pending():
                pendingJobs[job.get_id()] = job
        return pendingJobs
